#!/usr/bin/env python3
import RPi.GPIO as GPIO
import paho.mqtt.client as paho
import time
import json
import sys
import bios
import socket

def on_publish(client, userdata, mid):
    print("data published "+str(mid))
    pass 

def on_connect(client, userdata, flags, rc):
#    print("Connected with result code "+str(rc))
    subscribe_topics =[]
    for topic_tuple in mqtt_pub["gpio_port_2_topic"]:
        subscribe_topics.append(tuple((topic_tuple,0)))
        subscribe_topics.append(tuple((topic_tuple+str("/set"),0)))
        subscribe_topics.append(tuple((topic_tuple+str("/state"),0)))
    client.subscribe(subscribe_topics)
#    print(subscribe_topics)

# The callback for when a PUBLISH message is received from the server.
def on_message(client, userdata, msg):
#    print(str(msg.topic)+" "+str(msg.payload))
    for auto_gpio_port_item, auto_topic_item, auto_name_item, auto_class_item \
    in zip(mqtt_pub["gpio_port"], mqtt_pub["gpio_port_2_topic"], mqtt_pub["gpio_switch_name"], mqtt_pub["gpio_switch_class"]):
       if msg.topic==auto_topic_item+"/set" and msg.payload.decode('utf-8').upper() == "ON":
           GPIO.output(auto_gpio_port_item,1)     
#           print("ON = "+str(auto_gpio_port_item))
           (rc,mid) = client.publish(auto_topic_item+"/state", "ON", qos=1)
       if msg.topic==auto_topic_item+str("/set") and msg.payload.decode('utf-8').upper() == "OFF":
           GPIO.output(auto_gpio_port_item,0)
#           print("OFF = "+str(auto_gpio_port_item))
           (rc,mid) = client.publish(auto_topic_item+"/state", "OFF", qos=1)

mqtt_pub = bios.read('/etc/iot/binary_switches.yml')

client = paho.Client("RPi Binary Switches")
client.username_pw_set(mqtt_pub["mqtt"]["username"], mqtt_pub["mqtt"]["passwd"])
client.connect(mqtt_pub["mqtt"]["server"], mqtt_pub["mqtt"]["port"])
client.on_connect = on_connect
client.on_message = on_message
client.loop_start()

# Raspberry GPIO Configuration
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)

# grab hostname short
shorthost = socket.gethostname().split('.', 1)[0]

print("Reading MQTT Publish topic/s and set GPIO to states.")
for auto_gpio_port_item, auto_topic_item, auto_name_item, auto_class_item \
in zip(mqtt_pub["gpio_port"], mqtt_pub["gpio_port_2_topic"], mqtt_pub["gpio_switch_name"], mqtt_pub["gpio_switch_class"]):
    new_topic_item = auto_topic_item+"/config"
    fred = {"state_topic": auto_topic_item+"/state", "command_topic": auto_topic_item+"/set", "name": auto_name_item, \
        "unique_id": "rpi_gpio_"+str(auto_gpio_port_item), "retain": "true", "device": { "identifiers": [ "RPi GPIO Switch "+str(auto_gpio_port_item) ], \
        "name": "GPIO Circuit Switch on port "+str(auto_gpio_port_item)+" on host - "+str(shorthost)+", class:"+auto_class_item, \
        "model": "GPIO circuit switch on port "+str(auto_gpio_port_item)+" on host - "+str(shorthost), "manufacturer": "Raspberry Pi Foundation", "via_device": "RPi" }}
    # Raspberry GPIO Configuration
    GPIO.setup(auto_gpio_port_item, GPIO.OUT)
    (rc, mid) = client.publish(new_topic_item, json.dumps(fred), qos=mqtt_pub["qos"])
    (rc, mid) = client.publish(auto_topic_item+"/set", "OFF", qos=mqtt_pub["qos"], retain=True)
#    print(auto_gpio_port_item)
#    print(fred)
    if rc > 0:
        raise Exception(rc)

while True:
  try:
    for auto_gpio_port_item, auto_topic_item, auto_name_item, auto_class_item \
    in zip(mqtt_pub["gpio_port"], mqtt_pub["gpio_port_2_topic"], mqtt_pub["gpio_switch_name"], mqtt_pub["gpio_switch_class"]):
        state_topic_item = auto_topic_item
        if GPIO.input(auto_gpio_port_item)==True: 
#            print("state is true "+str(auto_gpio_port_item))
            (rc, mid) = client.publish(state_topic_item+"/state", "ON", qos=mqtt_pub["qos"])
        if GPIO.input(auto_gpio_port_item)==False:
#            print("state is false "+str(auto_gpio_port_item))
            (rc, mid) = client.publish(state_topic_item+"/state", "OFF", qos=mqtt_pub["qos"])
    if rc > 0:
        raise Exception(rc)

    time.sleep(mqtt_pub["interval"])
    
    for auto_gpio_port_item, auto_topic_item, auto_name_item, auto_class_item \
    in zip(mqtt_pub["gpio_port"], mqtt_pub["gpio_port_2_topic"], mqtt_pub["gpio_switch_name"], mqtt_pub["gpio_switch_class"]):
        new_topic_item = auto_topic_item+"/config"
        fred = {"state_topic": auto_topic_item+"/state", "command_topic": auto_topic_item+"/set", "name": auto_name_item, \
        "unique_id": "rpi_gpio_"+str(auto_gpio_port_item), "retain": "true", "device": { "identifiers": [ "RPi GPIO Switch "+str(auto_gpio_port_item) ], \
        "name": "GPIO Circuit Switch on port "+str(auto_gpio_port_item)+" on host - "+str(shorthost)+", class:"+auto_class_item, \
        "model": "GPIO circuit switch on port "+str(auto_gpio_port_item)+" on host - "+str(shorthost), "manufacturer": "Raspberry Pi Foundation", "via_device": "RPi" }}
        (rc, mid) = client.publish(new_topic_item, json.dumps(fred), qos=mqtt_pub["qos"])
        if rc > 0:
            raise Exception(rc)

  except Exception as e:
    print("There is a problem! The exception or return code is: ",e," - consult manuals for description")
    client.disconnect()
    sys.exit(rc)

GPIO.cleanup(gpiopin)
client.disconnect()
