#!/usr/bin/env python3
import paho.mqtt.client as paho
import time
import json
import sys
import bios
import re
import socket

def on_publish(client, userdata, mid):
#    print("data published "+str(mid))
    pass 

mqtt_pub = bios.read('/etc/iot/1w_temp_sensors.yml')

client = paho.Client("RPi 1 wire publish")
client.on_publish = on_publish
client.username_pw_set(mqtt_pub["mqtt"]["username"], mqtt_pub["mqtt"]["passwd"])
client.connect(mqtt_pub["mqtt"]["server"], mqtt_pub["mqtt"]["port"])
client.loop_start()

time.sleep(1)

# grab hostname short
shorthost = socket.gethostname().split('.', 1)[0]

print("Reading Device/s Temperature & Publishing to topic/s")
for auto_device_item,auto_topic_item,auto_name_item \
in zip(mqtt_pub["temperature_device_list"], mqtt_pub["temperature_topic_list"], mqtt_pub["temperature_name_list"]):
    new_topic_item = auto_topic_item+"/config"
    fred = {"state_topic": auto_topic_item, "json_attributes_topic": auto_topic_item, "unit_of_measurement": '°C', "value_template":\
"{{ value_json.temperature}}", "unique_id": auto_device_item, "name": auto_name_item, "device_class": "temperature", \
"device": { "identifiers": [ "RPi GPIO Sensor" ], "name": "Temperature Sensor "+auto_device_item+" device on host - "+str(shorthost), \
"model": "1 wire temperature sensor on host - "+str(shorthost) , "manufacturer": "Raspberry Pi Foundation" }}
    (rc, mid) = client.publish(new_topic_item, json.dumps(fred), qos=1)
    if rc > 0:
        raise Exception(rc)

while True:
  try:
    for device_item,topic_item in zip(mqtt_pub["temperature_device_list"], mqtt_pub["temperature_topic_list"]):
        # Open the file that we viewed earlier so that python can see what is in it. Replace the serial number as before.
        tfile = open("/sys/bus/w1/drivers/w1_slave_driver/"+device_item+"/w1_slave")
        # Read all of the text in the file.
        text = tfile.read()
        # Close the file now that the text has been read.
        tfile.close()
        # regex out the data and fix it up for celcius
        if (match := re.search('t= *([^\r\n]*)', text)) is not None:
            temperature = float(match.group(1))
            temperature = temperature / 1000
        y = { "temperature": str(round(temperature,2)) }
        (rc, mid) = client.publish(topic_item, json.dumps(y), qos=1)
        if rc > 0:
            raise Exception(rc)
    time.sleep(mqtt_pub["interval"])
    for auto_device_item,auto_topic_item,auto_name_item \
    in zip(mqtt_pub["temperature_device_list"], mqtt_pub["temperature_topic_list"], mqtt_pub["temperature_name_list"]):
        new_topic_item = auto_topic_item+"/config"
        fred = {"state_topic": auto_topic_item, "json_attributes_topic": auto_topic_item, "unit_of_measurement": '  C', "value_template":\
"{{ value_json.temperature}}", "unique_id": auto_device_item, "name": auto_name_item, "device_class": "temperature", \
"device": { "identifiers": [ "RPi GPIO Sensor" ], "name": "Temperature Sensor "+auto_device_item+" device on host - "+str(shorthost), \
"model": "1 wire temperature sensor on host - "+str(shorthost) , "manufacturer": "Raspberry Pi Foundation" }}
        (rc, mid) = client.publish(new_topic_item, json.dumps(fred), qos=1)
        if rc > 0:
            raise Exception(rc)

  except Exception as e:
    print("There is a problem! The exception or return code is: ",e," - consult manuals for description")
    client.disconnect()
    sys.exit(rc)
client.disconnect()
